// e952809.cpp (C) e952809@ecci.ucr.ac.cr

/** \file  e952809.cpp
    \brief Este programa sirve para constatar que varias implementaciones de
    \c strcpy() son correctas.

    \author Adolfo Di Mare <adolfo@di-mare.com>
    \date   2005

    - Enunciado ==> http://www.di-mare.com/adolfo/cursos/2005-1/p2-ta-1.htm
    - Why English names??? ==> http://www.di-mare.com/adolfo/binder/c01.htm#sc04
*/

#include "str_prtc.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/*
    Es mala idea copiar archivos '*.c' o '*.cpp' usando #include porque el nombre
    de ese archivo no puede aparecer en el archivo de proyecto del programa
    ( strcpy.cbp->Code::Blocks | strcpy.dev->Dev-C++ | strcpy.vcproj-Visual Studio )

    Al poner en el proyecto el archivo strcpy.c lo que ocurre es que todas las
    rutinas que contiene quedarn compiladas en 2 archivos diferentes:
    - e952809.cpp -> Este archivo, que contiene el programa principal main()
    - strcpy.c    -> archivo mencionado en el proyecto del programa C++
    Cuando el ligador de eslabonamiento (linker) encuentra que la misma rutina
    aparece en 2 lugares diferentes, emite un mensaje de error.

    Si el mensaje del linker no aparece, es porque esa versin es "inteligente"
    y es capaz de eliminar mdulos objeto duplicados. Esto es muy til cuando
    se usan plantillas, porque en cada archivo '*.cpp' en que son instanciadas
    el compilador produce todo el objeto completo para la implementacin de cada
    los mtodos de la clase emplnatillada que han sido usados.
*/
#include "strcpy.c" // MALO MALO MALO -- FEO FEO FEO

bool Verifica( const char* res );
void Dump( const char* name, const char * err_str, unsigned i, str_prtc& blq, const char* res);
void Mem_Dump(const char* V, unsigned Size);

#include "cassert" // assert()

/// Programa principal que contiene todos los datos de prueba
int main(int argc, char** argv) {
    Verifica("");
    Verifica("1");
    Verifica("1234");
    Verifica("ABCDE");
    Verifica("");

    return 0;
}

/** Ejecuta todas las versiones de \c strcpy() para constatar que funcionan
    correctamente con la hilera \c "res".
    - Verifica que el valor que queda almacenado en la hilera resulado
      sea igual a \c "res".
    - Si hay alguna divergencia, retorna \c "false".
    - Usa cada una de las implementaciones de \c "strcpy()" para realizar
      el trabajo.

    \returns \c "true" si encuentra algn error
*/
bool Verifica( const char* res ) {
    // Puntero a una funcin que implementa "strcpy()"
    char* (*strcpy[9]) (char *dst, const char *src) = {
        strcpy_1, strcpy_2, strcpy_3, strcpy_4, strcpy_5,
        strcpy_6, strcpy_7, strcpy_8, strcpy_9
    }; //  [ Stroustrup C++ | 7.7 Pointer to Function ]

    // Puntero a una funcin que implementa "strcpy()" CON 'typedef'
    typedef char* (*f_strcpy) (char *dst, const char *src);

    // tambin compila pero no lo uso
    f_strcpy strcpy_no[] = {
        strcpy_1, strcpy_2, strcpy_3, strcpy_4, strcpy_5,
        strcpy_6, strcpy_7, strcpy_8, strcpy_9
    };

    // cantidad de punteros almacenados en "f_strcpy[]"
    unsigned DIM_strcpy =  ( sizeof(strcpy) / sizeof(strcpy[0]) );
    assert(  DIM_strcpy == ( sizeof(strcpy) / sizeof(*strcpy) ));
    assert(  9 == DIM_strcpy );
    {
        /* DIM() no se puede implementar con plantillas... */
        #define DIM(V) (sizeof(V) / sizeof(*V)) // *V == *(V+0) == V[0]
        assert( 9 == DIM(strcpy) );
    }

    str_prtc dst, src;
    bool error_found = false;
    for ( unsigned i = 0; i < DIM_strcpy; ++i ) {
        dst.Fix(); src.Fix();
        dst = ""; // dst.operator= ( "" );
        src = res; // src.operator=( res );
        const char * returned = strcpy[i]( dst, src );
        bool err_1 = ! dst.Ok();
        bool err_2 = ( 0 != memcmp(dst, res, 1+strlen(res)) );
        bool err_3 = ( returned != dst );

        bool err_4 = ! src.Ok();
        bool err_5 = ( 0 != memcmp(src, res, 1+strlen(res)) );

        bool err   = (err_1 || err_2 || err_3) || (err_4 || err_5);
        char err_str[8]; err_str[0] = 0;

        if ( err_1 ) { strcat(err_str,"1"); }    if ( err_4 ) { strcat(err_str,"4"); }
        if ( err_2 ) { strcat(err_str,"2"); }    if ( err_5 ) { strcat(err_str,"5"); }
        if ( err_3 ) { strcat(err_str,"3"); }

        if ( err_1 || err_2 || err_3 ) { Dump( "dst", err_str, i+1, dst, res); }
        if ( err_4 || err_5 )          { Dump( "src", err_str, i+1, dst, res); }

        error_found = error_found || err;
    }
    return error_found;
}

/** Graba en la salida estndar el contenido completo del bloque de memoria \c "blq".

    - Los valores de la memoria quedan grabados en en hexagesimal
    - Esta rutina simplifica un poco la implementacin de \c Verifica()

    - Ejemplo con los valores siguientes:
      - <code> name    ==> "src"    </code>
      - <code> err_str ==> "246"    </code>
      - <code> i       ==> 4        </code>
      - <code> blq                  </code>
      - <code> res     ==> "alfa"   </code>
    - Resultado:

    \code
    ERROR[246]: strcpy_4(src,"alfa")
    src ==> :FF333333333333333333::31002121212121212121::00CCCCCCCCCCCCCCCCCC:
    \endcode
*/
void Dump( const char* name, const char * err_str, unsigned i, str_prtc& blq, const char* res) {
    void Mem_Dump(const char* V, unsigned SIZE); // declaracin previa de la funcin
    printf("\nERROR[%s", err_str);
    printf("]: strcpy_%d(%s,\"%s\")\n%s ==> ", i, name, res, name);
    {   Mem_Dump(blq.Before(), blq.SIZE_out);
        Mem_Dump(blq,          blq.SIZE_in);
        Mem_Dump(blq.After(),  blq.SIZE_out);
    }
}

#include <ctype.h>

/** Graba en la salida estndar el contenido de \c "V" en formato hexagesimal

    - Se supone que el tamao de \c "V" es \c "Size" caracteres
    - La cantidad de letras grabadas es <code> 2 * Size </code>
    - El valor queda grabado entre 2 delimitadores \c ':'
*/
void Mem_Dump(const char* V, unsigned Size) {
    printf(":");
    unsigned i;
    for (i = 0; i<Size; ++i) {
        unsigned hi = ( V[i] & 0x00F0 ) >> 4;
        unsigned lo =   V[i] & 0x000F;
        char chi = ( hi <= 9 ? hi+'0' : (hi-10)+'A' );
        char clo = ( lo <= 9 ? lo+'0' : (lo-10)+'A' );
        printf("%c%c", chi, clo );
    }
    printf(":");
}

// EOF: e952809.cpp
