// str_prtc.cpp (C) e952809@ecci.ucr.ac.cr

/** \file str_prtc.cpp
    \brief Implementacin de la clase \c str_prtc

    \author Adolfo Di Mare <adolfo@di-mare.com>
    \date   2005

    - Why English names??? ==> http://www.di-mare.com/adolfo/binder/c01.htm#sc04
*/

#include "str_prtc.h"

#include <cstring> // memset()
#include <cstring> // strlen()

/** Cambia el valor de la hilera \c "*this" para que contenga la hilera \c "res".

    -  Slo copia la parte de la hilera \c "res" que cabe en \c "*this"
*/
const char* str_prtc::operator=( const char* res ) {
    memset( m_s, Fix_char, SIZE_in ); // blanque "*this"
    unsigned resm_size = 1+strlen( res ); // incluye el EOS
    if (resm_size <= SIZE_in) { // slo asigna si la hilera cabe
        memcpy( m_s, res, resm_size );
    } else {
        memcpy( m_s, res, SIZE_in );
        m_s[ SIZE_in - 1 ] = 0; // trunca
    }
    return m_s;
}

/** Corrige el valor del objeto \c "*this" si no cumple con su invariante.

   - En muchas ocasiones \c Fix() puede implementarse como un constructor,
     pero puede que esto no sea ni lo mejor o ni lo menos costoso.
   - Para muchas objetos o clases es imposible implementar esta operacin.
   - Si esta es una operacin de la clase, lo razonable es que tambin lo
     sea \c Ok().

    \see http://www.di-mare.com/adolfo/binder/c04.htm#sc12
*/
void str_prtc::Fix() {
    if ( Ok() ) { // Si Fix() est implementada, tambin lo est Ok()
        return;
    }
    unsigned i;

    m_after[0] = 0; // pone un EOS==0 al final de la hilera "m_s"
    for (i = 0; i < SIZE_out; ++i) {
        m_before[i] = ((char)0xFF ^ m_after[i]);
    }

    for (i = 0; i < sizeof(str_prtc::m_s); ++i) {
        m_s[i] = Fix_char; // llena toda la hilera "m_s" de "chingolitos"
    }
}

#include <cassert> // assert()

/**   Verifica que se cumpla la invariante de la clase, o sea,
      que el objeto est bien construido

      - La razn por la que esta funcin no es un mtodo es continuar la
        costumbre de muchos programadores quienes no definen la invariante para
        sus clases, pues en muchos casos sobra hacerlo.
      - Esta funcin en general es difcil de implementar, y en algunos casos
        es imposible pues, cuando el objeto no est bien construido, puede
        ocurrir que la funcin no retorne.
      - En general, la implementcin de esta funcin no es completa pues hay
        casos en que es imposible verificar la invariante de una clase.

         \post Retorna \c "true" si el la hilera es un objeto bien construido

    \see http://www.di-mare.com/adolfo/binder/c04.htm#sc11
*/
bool Check_Ok(const str_prtc& STR) {
    unsigned i;
    if (STR.m_after[0] != 0) { // verifica EOS al final de la hilera
//          i = i;
        return false;
    }

    for (i = 0; i < str_prtc::SIZE_out; ++i) {
        if (STR.m_before[i] != ((char)0xFF ^ STR.m_after[i])) {
            return false; // verifica que los [amortiguadores] alrededor
        }                 // de la hilera no estn [manchados]
    }

    i = str_prtc::SIZE_in;
    do {
        i--;
        if (0 == STR.m_s[i]) { // encontr EOS y todo est bien
            break;
        } else if (STR.m_s[i] != str_prtc::Fix_char) {
            return false; // se queja porque no hay [chingolitos] despus de la hilera
        }
    } while (i != 0);
    if ( i == 0 ) {
        return true;
    }
    // assert( i > 0 );
    do {
        i--;
        if (0 == STR.m_s[i]) {
            return false; // encontr ms de un EOS
        }
    } while (i != 0);

    return true;
}

// EOF: str_prtc.cpp
