// str_prtc.h   (C) e952809@ecci.ucr.ac.cr

/** \file str_prtc.h
    \brief Declaraciones para la clase \c str_prtc

    \author Adolfo Di Mare <adolfo@di-mare.com>
    \date   2005

    - Why English names??? ==> http://www.di-mare.com/adolfo/binder/c01.htm#sc04
*/

#ifndef str_prtc_h
#define str_prtc_h

/** \brief Esta clase incluye un arreglo de letras para almacenar una hilera.

    - Alrededor del valor almacenado hay 2 grandes bloques, que sirven para
      determinar si, por una falla del programa, se ha sobreescrito memoria
      antes o despus de la hilera almacenada.

    \code
    +--------------+--------------+--------------+
    |F(/"$'?);)=:~!| << hilera >> |0(/"$'?);)=:~!|
    +--------------+--------------+--------------+
    \endcode
*/
class str_prtc {
public:  // use 'enum{}' si el compilador no le permite usar 'const int'
    enum { SIZE_in  =          10, ///< Tamao mximo de una hilera
           SIZE_out = 1 * SIZE_in, ///< Tamao del amortiguador alrededor del valor de la hilera
           Fix_char = '!'          ///< Letra de relleno al final de la hilera [chingolito]
    };
private:
    char m_before [SIZE_out]; ///< Memoria [amortiguador] "antes" de la hilera
    char m_s      [SIZE_in];  ///< Valor de la hilera
    char m_after  [SIZE_out]; ///< Memoria [amortiguador] "despus" de la hilera

public:
    str_prtc() { Fix(); } ///< Constructor de vector
    friend bool Check_Ok(const str_prtc&);
    bool   Ok() { return Check_Ok(*this); } ///< Usa \c Check_Ok() para verificar la invariante de la clase
    void   Fix();
    const char* operator=( const char* );
    operator char* ( ) { return m_s; } ///< Retorna el valor almacenado en la hilera

public:
    const char* Before() { return m_before; } ///< Referencia a la memoria que est antes de la hilera
    const char* After()  { return m_after;  } ///< Referencia a la memoria que est despus de la hilera
}; // str_prtc

#endif // str_prtc_h

// EOF: str_prtc.h
