/* strcpy.c   (C) adolfo@di-mare.com */

/** \file strcpy.c
    \brief Implementaciones para \c strcpy.c()

    Contiene varias implementaciones diferentes de strcpy() para
    ilustrar la equivalencia de punteros/vectores en C/C++
    - Ref: pp.325, Deitel & Deitel

    \author Adolfo Di Mare <adolfo@di-mare.com>
    \date   2005
*/

/** Copia en \c "dst" el valor de la hilera \c "src".

    \pre \c "dst" debe ser suficientemente grande.

    \returns \c "dst"
*/
char* strcpy (char *dst, const char *src);

#if 0
/** Copia en \c "dst" el valor de la hilera \c "src".
    - \c "dst" debe ser suficientemente grande.
    - Retorna \c "dst".
    \par Cmo
     Copia las letras que aparecen a partir de \c "*src" en
    \c "*dst" hasta que copia el valor (char*) 0.
*/
char* strcpy( char * dst, const char * src );
#endif

char* strcpy_1 (char *dst, const char *src) {
    int i = -1;
    do {
        i++;
        dst[i] = src[i];
    } while ( src[i] != 0 );

    return & dst[0];
}

char* strcpy_2 (char *dst, const char *src) {
    unsigned int i = 0;
    dst[i] = src[i];
    while ( src[i] != 0 ) {
        dst[i] = src[i];
        i++;
    }

    return & dst[0];
}

char* strcpy_3 (char *dst, const char *src) {
    unsigned int i = 0;
    while ( (dst[i] = src[i]) != 0 ) {
        i++;
    }

    return & dst[0];
}

char* strcpy_4 (char *dst, const char *src) {
    unsigned int i = 0;
    while ( ( *(dst+i) = *(src+i) ) != 0 ) {
        i++;
    }

    return & dst[0];
}

char* strcpy_5 (char *dst, const char *src) {
    char       * p;
    const char * q;

    p = & dst[0];
    q = & src[0];

    while ( (*p = *q ) != 0 ) {
        p++; q++;
    }

    return dst;
}

char* strcpy_6 (char *dst, const char *src) {
    char       * p;
    const char * q;

    p = dst;
    q = src;

    while ( (*p = *q ) != 0 ) {
        p++; q++;
    }

    return dst;
}

char* strcpy_7 (char *dst, const char *src) {
    char       * p;
    const char * q;

    for ( p = dst, q = src; (*p = *q ) != 0 ; (p++, q++) ) { }

    return dst;
}

char* strcpy_8 (char *dst, const char *src) {
    char       * p;
    const char * q;

    for ( p = dst, q = src; (*p = *q); (p++, q++) ) { }

    return dst;
}

char* strcpy_9 (char *dst, const char *src) {
    char       * p;
    const char * q;

    for ( p = dst, q = src; (*p++ = *q++); ) { }

    return dst;
}

/* EOF: strcpy.c */
